;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;	NeoGeo CD demo by Charles Doty (cdoty@netzero.net)
;;;;
;;;;	Some stuff to remember when you write in VRAM
;;;;
;;;;	write at 0x7000 to 0x7a00
;;;;	1 WORD		pppp ssss ssss ssss	where p = palette number and
;;;;						s = sprite/tile number
;;;;	Tiles are 8x8 and come from FIX rom
;;;;	7000 is (0,0)
;;;;	7001 is (0,1)
;;;;	701F is (0,31)
;;;;	7020 is (1,0)
;;;;			so for a X+1, you need to write at adr+0x20
;;;;
;;;;	write at 0x00000 to 0xDFFFF
;;;;	1 DWORD		0xSSPF	where S = sprite/tile number,
;;;;				P = palette number and F=flip and special
;;;;	Tiles are 16x16 (???? 4 tiles of 8x8 ????) and come from GFX rom
;;;;	00000 is (0,0)
;;;;	00002 is (0,1)
;;;;	0003E is (0,31)
;;;;	00040 is (1,0)
;;;;			so for a X+1, you need to write at adr+0x40
;;;;
;;;;	write at 0x8000,0x8200,0x8400 to 0x81FF,0x83FFF,0x85FFF
;;;;	1 WORD		see neogeo doc
;;;;
;;;;	0x8?00 is for sprite block 0
;;;;	0x8?02 is for sprite block 1
;;;;
;;;;	What i don't understand is the link between 0x8000 and 0x00000
;;;;	is it
;;;;	0x8000 point to 0x00000
;;;;	0x8002 point to 0x00080
;;;;	.....????
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	INCLUDE	Includes/Defines.inc

	ORG		$0000

	dc.l	 STACKPTR,$00C00402,$00C00408,$00C0040E	; Stack Ptr = 0x0010F300
													; InitialPC = 0x00C00402
	dc.l	$00C00414,$00C00426,$00C00426,$00C00426	; Ptr to debugging switch = 0x00C00414
	dc.l	$00C0041A,$00C00420,$00C00426,$00C00426
	dc.l	$00C00426,$00C00426,$00C00426,$00C0042C
	dc.l	$00C00522,$00C00528,$00C0052E,$00C00534
	dc.l	$00C0053A,$00C004F2,$00C004EC,$00C004E6
	dc.l	$00C00432,     Irq1,   VBlank,     Irq3	; Vertical blank ptr=VBlank
    						 						; IRQ2 ptr=Irq2(see below)
	dc.l 	  Generic,  Generic,  Generic,  Generic
	dc.l	  Generic,  Generic,  Generic,  Generic
	dc.l	  Generic,  Generic,  Generic,  Generic
	dc.l	  Generic,  Generic,  Generic,  Generic
	dc.l	  Generic,  Generic,  Generic,  Generic
	dc.l	$00C00426,$00C00426,$00C00426,$00C00426
	dc.l	$00C00426,$00C00426,$00C00426,$00C00426
	dc.l	$00C00426,$00C00426,$00C00426,$00C00426
	dc.l	$00C00426,$00C00426,$00C00426,$00C00426

	dc.b	"NEO-GEO", $00							; Magic bytes
	dc.w	$0202									; 0x0202 = NGH number
	dc.l	$00000000							    ; Unknown
	dc.l	$010E000								; Pointer to debugging switch
	dc.w	$0000									; Blank
	dc.w	$0100									; Starting sprite number = 0x0100

	dc.l	JapaneseConfig							; Pointer to Japanese config
	dc.l	EnglishConfig							; Pointer to English config
	dc.l	SpanishConfig							; Pointer to Spanish config

	jmp		Start									; Entry point of program
	jmp		Start2
	jmp		Start3
	jmp		Start4

; Generic interrupt routine
Generic:
	rte
	rts

	dc.w	$FFFF

JapaneseConfig:
	dc.b	"NEOGEO DEMO JP  "
	dc.l	$FFFFFFFF
	dc.w	$0364
	dc.b	$14, $13, $24, $01

EnglishConfig:
	dc.b	"NEOGEO DEMO EN  "
	dc.l	$FFFFFFFF
	dc.w	$0364
	dc.b	$14, $13, $24, $01

SpanishConfig:
	dc.b	"NEOGEO DEMO SP  "
	dc.l	$FFFFFFFF
	dc.w	$0364
	dc.b	$14, $13, $24, $01

VertBlank:
	dc.b	0

Direction:
	dc.b	0

Flip:
	dc.b	0
	dc.b	0

; Irq1 interrupt routine
Irq1:
	move.w	#2, $3C000C			; IRQ 2 acknoledge
	rte

; VBlank interrupt routine
VBlank:
	tst.b	$10FD80				; Bit 7 = 1 when NeoGeo logo is displayed
	bmi		DisplayedLogo

	jmp		$C00438				; Bios logo display function

DisplayedLogo:
	move.b	d0, $300001			; Watchdog
	move.b	#1, VertBlank		; Vertical Blank occured

	jsr		$C0044A

	move.w	#4, $3C000C			; IRQ 2 acknoledge?

	rte

; Irq3 interrupt routine
Irq3:
	move.w	#1, $3C000C			; IRQ 1 acknoledge
	rte

; Entry point
Start:
	lea		STACKPTR, sp

	jsr		Setup

; Write Message text
	lea		Message, a2
	move.w	#15, d1
	move.w	#36, d0

	move.w	#1, d3				; X Position of 10
	asl.l	#5, d3				; x *= 0x20 (since +1 = +0x20)
	add.w	#10, d3				; Y Position
	add.w	#$7000, d3			; Sprite Offset    0x7000 is the FIX plane
	move.w	d3, d4
	move.w	#$0000, d6

MessageLoop:
	move.w	d3, $3c0000			; Set write location in VRAM
	move.w	#$3000, d7			; Palette 3 and index base = 0x00
								; (font starts at 0x00 in FIX area)
	move.b	(a2)+, d6
	add.w	d6, d7
	move.w	d7, $3c0002
	add.l	#$20, d3			; x + 1
	dbra	d0, MessageLoop		; Do each column

	move.w	#36, d0
	addq	#1, d4				; y + 1
	move.l	d4, d3
	dbra	d1, MessageLoop		; Do each line

; Same thing for Members text
	lea		Members, a2
	move.w	#3, d1
	move.w	#36, d0

	add.w	#1, d4
	move.l	d4, d3

MembersLoop:
	move.w	d3, $3c0000
	move.w	#$3000, d7			; Palette number
	move.b	(a2)+, d6
	add.w	d6, d7
	move.w	d7, $3c0002
	add.l	#$20, d3
	dbra	d0, MembersLoop

	move.w	#36, d0
	addq	#1, d4
	move.l	d4, d3
	dbra	d1, MembersLoop

	move.w	#$8010, $3c0000		; Start write at 0x8010
	move.w	#$200,  $3c0004		; Increment VRAM pos by 0x200
	move.w	#$0fff, $3c0002		; 0x8010 = Full size for sprite
	move.w	#$fc0e, $3c0002		; Normally $f80e
								; 0xE sprite in the bank
								; Ypos = 0xFC0E >> 7 = 5004
	move.w	#$440, $3c0002		; Xpos = 0x440 >> 7 = 8

	move.w	#$8011, d1
	move.w	#18, d0				; Sprite width - 1

SpriteMapLoop:
	move.w	d1, $3c0000
	move.w	#$0fff, $3c0002		; Full size/zoom
	move.w	#$40, $3c0002		; Bit 6 = 1 : next sprite at right
	add.w	#1, d1
	dbra	d0, SpriteMapLoop

	move.w	#$1, $3c0004		; VRAM increment by 1
	lea		ChaosMap, a5
	move.w	#$0400, a0
	move.w	a0, a1

	move.w	#3, d4

ChaosOuterLoop:
	move.w	#19, d5

ChaosInnerLoop:
	move.w	a0, $3c0000			; Location to write a0
	move.w	(a5)+, $3c0002		; First word = sprite number
	move.w	#$0000, $3c0002		; Second word = palette(0) and flip(0)
	add.w	#64, a0				; X + 1
	dbra.w	d5, ChaosInnerLoop

	add.w	#2, a1				; Y + 1
	move.w	a1, a0

	dbra.w	d4, ChaosOuterLoop

; Same thing
	lea		TilesMap, a5

	move.w	#11, d4

TilesOuterLoop:
	move.w	#18, d5

TilesInnerLoop:
	move.w	a0, $3c0000
	move.w	(a5)+, $3c0002
	move.w	#$0100, $3c0002
	add.w	#64, a0
	dbra.w	d5, TilesInnerLoop

	add.w	#2, a1
	move.w	a1, a0

	dbra.w	d4, TilesOuterLoop


	move.w	#$8100, $3c0000		; Write at $8100
	move.w	#$200,  $3c0004		; Inc by $200
	move.w	#$0FFF, $3c0002		; Full size
	move.w	#$F20E, $3c0002		; Normally 0xF80E
								; 0x0E sprite in the bank
								; Y = 0xF20E >> 7
	move.w	#$2800, $3c0002		; X = 0x2800 >> 7

	move.w	#$8101, d1
	move.w	#8, d0

MovingMapLoop:
	move.w	d1, $3c0000
	move.w	#$0FFF, $3c0002		; Full size
	move.w	#$40, $3c0002		; Bit 6 = 1   (next tile right)
	add.w	#1, d1
	dbra	d0, MovingMapLoop	; From $8101 to $8109

	move.w	#$01,  $3c0004		; Increment by 1
	move.w	#$4000, a0
	move.w	#93, d0
	move.w	#103, d1
	move.w	#9, d5

MovingLoop:
	move.w	a0, $3c0000			; Write to (a0)

	move.w	d0, $3c0002			; Sprite number = d0
	move.w	#$0200, $3c0002		; Palette 2 & no flip

	add.w	#1, d0

	move.w	d1, $3c0002			; Sprite number = d1
	move.w	#$0200, $3c0002		; Palette 2 & no flip

	add.w	#1, d1

	move.w	#206, $3c0002		; Sprite number = 206
	move.w	#$0200, $3c0002		; Pallete  & no flip

	move.w	#206, $3c0002
	move.w	#$0200, $3c0002

	move.w	#206, $3c0002
	move.w	#$0200, $3c0002

	move.w	#206, $3c0002
	move.w	#$0200, $3c0002

	move.w	#206, $3c0002
	move.w	#$0200, $3c0002

	move.w	#206, $3c0002
	move.w	#$0200, $3c0002

	move.w	#206, $3c0002
	move.w	#$0200, $3c0002

	move.w	#206, $3c0002
	move.w	#$0200, $3c0002

	move.w	#206, $3c0002
	move.w	#$0200, $3c0002

	move.w	#206, $3c0002
	move.w	#$0200, $3c0002

	move.w	#206, $3c0002
	move.w	#$0200, $3c0002

	move.w	#206, $3c0002
	move.w	#$0200, $3c0002

	move.w	#206, $3c0002
	move.w	#$0200, $3c0002

	move.w	#206, $3c0002
	move.w	#$0200, $3c0002

	add.w	#$40, a0			; X + 1

	dbra.w	d5, MovingLoop

	move.b	#0, VertBlank		; Reset Vertical Blank
	move.b	#0, Direction		; Init values
	move.b	#0, Flip			; Chaos logo is not flipped

	move.w	#$2000, sr			; Enable interrupts

Loop:
	cmp.b	#1, VertBlank		; Wait sync
	bne		Loop

	move.b	$300000, d0			; d0 = controler 1 status
	not.b	d0					; Invert bits <= DON'T FORGET THIS

	btst	#2, d0				; Bit 2 = Left  so if bit 2 = 1 then NotLeft
	beq		NotLeft

	; Move left
	move.w	#$8500, $3c0000
	move.w	$3c0002, d1			; Read current X

	cmp.w	#128, d1			; 1 * 128 (128 = 1 << 7)
	ble		EndInput

	sub.w	#128, d1			; If x > 1, x-1

	move.w	#$8500, $3c0000		; Write new X
	move.w	d1, $3c0002

	bra		EndInput

NotLeft:
	btst	#3, d0				; Bit 3 = Right so if bit 3 = 1 then NotRight
	beq	EndInput

	; Move right
	move.w	#$8500, $3c0000		; Read curr x
	move.w	$3c0002, d1

	cmp.w	#20480, d1  		; 160 * 128 (160 = 320 - Width*TileWidth
								; = 320 - 10 * 16 = 320 - 160)
	bge		EndInput

	add.w	#128, d1			; If (x + width) < 320, x+1

	move.w	#$8500, $3c0000		; Write new x
	move.w	d1, $3c0002

EndInput:
	move.b	#0, VertBlank		; Reset VBlank status

	cmp.b	#1, Direction
	beq		GoingDown

	; Rotate up
	move.w	#$8100, $3c0000
	move.w	$3c0002, d1			; Read current zoom
	move.w	d1, d3
	and.w	#$00ff, d1			; d1 = Y zoom
	and.w	#$ff00, d3			; d3 = X zoom

	cmp.w	#$00, d1			; If Zoom = 0
	beq		ResetDown

	sub.w	#$0f, d1			; Y zoom - 1

	move.w	#$8100, $3c0000

	or.w	d3, d1				; Restore x zoom
	move.w	d1, $3c0002			; Write new zoom

	move.w	#$8300, $3c0000		; Center the sprite
	move.w	$3c0002, d1

	sub.w	#128, d1

	move.w	#$8300, $3c0000
	move.w	d1, $3c0002

	jmp		Repeat

; Change direction
ResetDown:
	move.b	#1, Direction
	move.w	#$8100, $3c0000

	or.w	d3, d1
	move.w	d1, $3c0002

	jsr		FlipLogo

	jmp		Repeat

GoingDown:						; Same thing but with y zoom + 1
	move.w	#$8100, $3c0000
	move.w	$3c0002, d1
	move.w	d1, d3
	and.w	#$00ff, d1
	and.w	#$ff00, d3

	cmp.w	#$FF, d1
	beq		ResetUp

	add.w	#$0f, d1			; Y zoom + 1

	move.w	#$8100, $3c0000

	or.w	d3, d1
	move.w	d1, $3c0002

	move.w	#$8300, $3c0000		; Center the sprite
	move.w	$3c0002, d1

	add.w	#128, d1

	move.w	#$8300, $3c0000
	move.w	d1, $3c0002

	jmp		Repeat

ResetUp:
	move.b	#0, Direction
	move.w	#$8100, $3c0000

	or.w	d3, d1
	move.w	d1, $3c0002

Repeat:
	bra		Loop

Start2:
	rts

Start3:
	rts

Start4:
	rts

; Initialize
Setup:
	move.w	#0, $3c0006			; ???
	move.w	#7, $3c000c			; IRQ 1,2,4 acknoledge

	jsr		SetupPalette		; Transfer palettes

	jsr		$c004c2				; Call BIOS routine

	movem.l d0-d6, -(a7)		; Save data registers

	move.w	#$7000, d0			; d0 = (0, 0)
	move.w	#$701f, d1			; d1 = (0, 31)
	move.w	#$7001, d4			; d4 = (0, 1)
	move.w	#$701e, d5			; d5 = (0, 30)
	move.w	#$20, d2
	move.w	#$20, d3
	move.w	#$27, d7

SetupLoop:
	move.w	d0, $3c0000
	nop
	move.w	d2, $3c0002
	nop
	move.w	d1, $3c0000
	nop
	move.w	d2, $3c0002
	nop
	move.w	d4, $3c0000
	nop
	move.w	d2, $3c0002
	nop
	move.w	d5, $3c0000
	nop
	move.w	d2, $3c0002
	nop

	add.w	d3, d0				; d0 + 0x20 = X0 + 1
	add.w	d3, d1				; X1 + 1
	add.w	d3, d4				; X4 + 1
	add.w	d3, d5				; X5 + 1
	dbra	d7, SetupLoop		; Do 40 (0x27 ie 320/8) columns

	movem.l	(a7)+, d0-d6		; Restore data registers

	jsr		$c004c8 			; Call bios routine

	rts

; Initialize the palette
SetupPalette:
	lea		$400000, a0		; A0 = ptr to NeoGeo palette
	lea		Palettes, a1	; A1 = ptr to our palette
	move.w	#EndPalettes-Palettes, d7	; D7 = palette size
	asr.w	#5, d7			; Each pal = 16 words so palette number = d7 >> 5

; Copy 16 color per palettes
DoPalette:
	move.w	(a1)+, (a0)+
	move.w	(a1)+, (a0)+
	move.w	(a1)+, (a0)+
	move.w	(a1)+, (a0)+
	move.w	(a1)+, (a0)+
	move.w	(a1)+, (a0)+
	move.w	(a1)+, (a0)+
	move.w	(a1)+, (a0)+
	move.w	(a1)+, (a0)+
	move.w	(a1)+, (a0)+
	move.w	(a1)+, (a0)+
	move.w	(a1)+, (a0)+
	move.w	(a1)+, (a0)+
	move.w	(a1)+, (a0)+
	move.w	(a1)+, (a0)+
	move.w	(a1)+, (a0)+
	dbra	d7, DoPalette

	rts

FlipLogo:
	move.w	#1, $3c0004		; Increment vram by 1

	move.w	#$4000, a0
	move.w	#9, d5

	move.b	Flip, d0
	add.b	#1, d0
	and.b	#1, d0
	move.b	d0, Flip		; Flip = 1 OR 0

	cmp.b	#1, d0
	bne		Normal			; Flip 0 = normal

	move.w	#103, d0		; Flip
	move.w	#93, d1
	move.w	#$0202,d2		; Palette 2, y flip

	jmp		FlipLoop

Normal:
	move.w	#93, d0		; No flip
	move.w	#103, d1
	move.w	#$0200,d2		; Palette 2, no flip


; Write new values
FlipLoop:
	move.w	a0, $3c0000

	move.w	d0, $3c0002
	move.w	d2, $3c0002

	add.w	#1, d0

	move.w	d1, $3c0002
	move.w	d2, $3c0002

	add.w	#1, d1
	add.w	#$40, a0

	dbra	d5, FlipLoop

	rts

;            *         1         2         3     *
;            0123456789012345678901234567890123456
Message:
	dc.b	"   WELCOME TO THE 10TH ANNIVERSARY   "
	dc.b	" CHAOS DEMO, AND FIRST EVER NEOGEO   "
	dc.b	" DEMO. THIS THE SECOND CHAOS DEMO ON "
	dc.b	" AN ARCADE GAME SYSTEM. THE CPS-1    "
	dc.b	" SYSTEM IS THE NEXT PLANNED SYSTEM TO"
	dc.b	" BE BLESSED WITH A CHAOS RELEASE. :) "
	dc.b	"                                     "
	dc.b	" NOW TO GET A HOLD OF A JAMMA RIG TO "
	dc.b	" TEST THE DEMOS ON A REAL SYSTEM.    "
	dc.b	"                                     "
	dc.b	" EVENTUALLY WE RE PLANNING ON DOING  "
	dc.b	" SOME REAL ARCADE GAMES.             "
	dc.b	"                                     "
	dc.b	" CONTACT RASTER AT CDOTY@NETZERO.NET "
	dc.b	"                                     "
	dc.b	" LONG LIVE POOPER*****CODED BY RASTER"

Members:
	dc.b	"              MEMBERS:               "
	dc.b	"     RASTER            PICKSTER      "
	dc.b	"           SECTOR SMASHER            "

SpriteNumber:
	dc.w	14					; Temp

Palettes:
	INCBIN	"chaos.pal"
	INCBIN	"tiles.pal"
	INCBIN	"sprite.pal"
	INCBIN	"font.pal"
EndPalettes:

ChaosMap:
	INCBIN	"chaos.mp1"
ChaosMapEnd:

TilesMap:
	INCBIN	"tiles.mp1"
TilesMapEnd:

Filler:
	ds.b	$80000-Filler

	END
