;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;	NeoGeo WW2 by Charles Doty (cdoty@netzero.net)
;;;;	Routines.asm - Contains useful Routines.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SetupTilemap:
	movem.l d0-d1, -(sp)		; Save data registers

	move.w	#SPRITETILE, d1
	move.w	d1, VRAM_ADDR		; Starting tile number
	move.w	#$200,  VRAM_INCR	; Increment VRAM pos by 0x200
	move.w	#FULLZOOM, VRAM_DATA; 0x0FFF = Full size for sprite
	move.w	#TILEY, VRAM_DATA	; Starting y position of the tilemap
								; 0xE sprite in the bank
								; Ypos = 0xFC0E >> 7 = 5004
	move.w	#$440, VRAM_DATA	; Xpos = 0x440 >> 7 = 8

	move.w	#18, d0				; Sprite width - 1

SetupTilemapLoop:
	add.w	#1, d1
	move.w	d1, VRAM_ADDR
	move.w	#FULLZOOM, VRAM_DATA; Full size/zoom
	move.w	#$40, VRAM_DATA		; Bit 6 = 1 : next sprite at right
	dbra	d0, SetupTilemapLoop

	movem.l (sp)+, d0-d1		; Restore data registers

	rts

; A0 - VRAM Location
; A1 - Map pointer
; D0 - Width
; D1 - Height
FillTilemap:
	movem.l a0-a2/d0-d4, -(sp)	; Save data registers

	clr.l	d2
	move.w	ScrollPos, d2       ; Load scroll position
	lsr.l	#4, d2				; Divide by 16 to affect only tiles
	lsl.l	#2, d2				; Multiply by 4 for 4 bytes per map location
	add.l	d2, a1

	move.l	a1, a2
	move.w	#$01, VRAM_INCR		; VRAM increment by 1
	move.w	a0, VRAM_ADDR		; Location to write a0
	move.w	d1, d2				; Save Height

FillTilemapOuterLoop:
	move.w	d2, d1

FillTilemapInnerLoop:
	move.w	(a1)+, VRAM_DATA	; Transfer tile number
	move.w	(a1)+, VRAM_DATA	; Transfer attribute

	dbra.w	d1, FillTilemapInnerLoop

   	add.l	#MAPHEIGHT*4, a2	; Skip to next collumn
	move.l	a2, a1
	add.w	#64, a0				; X + 1
	move.w	a0, VRAM_ADDR		; Location to write a0

	dbra.w	d0, FillTilemapOuterLoop

	move.w	#SPRITETILE, d0
	add.w	#$200, d0

	move.w	d0, VRAM_ADDR

	move.w	ScrollPos, d1
	and.w	#$0F, d1
	lsl.w	#7, d1

	move.w	VRAM_DATA, d2
	and.w	#$007F, d2
	or.w	d1, d2

	move.w	d0, VRAM_ADDR
	move.w	d2, VRAM_DATA

	movem.l (sp)+, a0-a2/d0-d4	; Restore data registers

	rts

; Initialize the palette
SetupPalette:
	movem.l	d0/a0-a1, -(sp)		; Restore data registers

	asr.w	#5, d0				; Each pal = 16 words so palette number = d0 >> 5

SetupPaletteLoop:				; Copy 16 color per palettes
	move.w	(a1)+, (a0)+
	move.w	(a1)+, (a0)+
	move.w	(a1)+, (a0)+
	move.w	(a1)+, (a0)+

	move.w	(a1)+, (a0)+
	move.w	(a1)+, (a0)+
	move.w	(a1)+, (a0)+
	move.w	(a1)+, (a0)+

	move.w	(a1)+, (a0)+
	move.w	(a1)+, (a0)+
	move.w	(a1)+, (a0)+
	move.w	(a1)+, (a0)+

	move.w	(a1)+, (a0)+
	move.w	(a1)+, (a0)+
	move.w	(a1)+, (a0)+
	move.w	(a1)+, (a0)+

	dbra	d0, SetupPaletteLoop

	movem.l (sp)+, d0/a0-a1		; Restore data registers

	rts

ReadController:
	move.b	Joy1, d0

	rts
